/*
 * dvp_hbi.h
 *
 * This file declares the VCP Host Bus Interface layer.
 *
 * Copyright (c) 2008, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */


#ifndef _VCP_HBI_H
#define _VCP_HBI_H

#include "vp_hal.h"

#ifndef	VP_VCP_MAX_NUM_LINES
#define VP_VCP_MAX_NUM_LINES           (32)	/* Maximum lines handled by VCP */
#endif /* VP_VCP_MAX_NUM_LINES */

/*
 * The following constants define the HBI page, offset, length, and number of
 * instances of all registers exposed by the VCP's Host Bus Interface.
 * Object lengths are given in 16-bit words.
 *
 * The various VP data elements are arranged in the HBI under the following
 * rules:
 *  1)  Although the VCP page select register is 8-bits wide, theoretically
 *      allowing up to 256 pages to be addressed, only pages 0-15 are currently
 *      implemented on the VCP.
 *  2)  7-bits of offset means that there are only 128 locations per page
 *		(VP_HBI_PAGE_SIZE = 0x80).
 *  3)  VP DATA ELEMENTS THAT MAY BE ACCESSED SEQUENTIALLY AS A BLOCK MUST NOT
 *      SPAN ACROSS PAGE BOUNDARIES!!! This requirement stems from the fact that
 *      VCP "paged offset" accesses beyond the 128th word in a page are ignored.
 *  4)  The direct page is used to hold data that must be quick or easy to
 *      access. The location of fixed hardware registers within the direct page
 *      must be accounted for.
 *
 * Note that the mapping of paged HBI memory to physical VCP memory is solely
 * determined by the VCP software. Also, VCP software can configure any page to
 * be shorter than 128 words.
 *
 * DevNotes:
 *  could easily check for violations of page boundaries in hbi hal functions
 *  could test for code not matching hw in main...
 */
#define VP_HBI_PAGE_SIZE            0x80

/* AC profiles */
#define VP_REG_AC_PROF_PAGE(n)      n
#define VP_REG_AC_PROF_OFFSET(n)    0x54
#define VP_REG_AC_PROF_LENGTH       44
#define VP_REG_AC_PROF_COUNT        3

/* Cadence profiles */
#define VP_REG_CAD_PROF_PAGE(n)     (6 + (n / 6))
#define VP_REG_CAD_PROF_OFFSET(n)   (19 * (n % 6))
#define VP_REG_CAD_PROF_LENGTH      19
#define VP_REG_CAD_PROF_COUNT       15

/* Caller ID profiles */
#define VP_REG_CID_PROF_PAGE(n)     ((n < 2) ? (n + 1) : ((n + 15) >> 1))
#define VP_REG_CID_PROF_OFFSET(n)   (((n == 0) | (n & 1)) ? 0 : 0x40)
#define VP_REG_CID_PROF_LENGTH      64
#define VP_REG_CID_PROF_COUNT       10

/* DC profiles */
#define VP_REG_DC_PROF_PAGE(n)      0
#define VP_REG_DC_PROF_OFFSET(n)    (0x27 + 15 * n)
#define VP_REG_DC_PROF_LENGTH       15
#define VP_REG_DC_PROF_COUNT        3

/* Device profile */
#define VP_REG_DEV_PROF_PAGE(n)     0
#define VP_REG_DEV_PROF_OFFSET(n)   0
#define VP_REG_DEV_PROF_LENGTH      20
#define VP_REG_DEV_PROF_COUNT       1

/* Metering profiles */
#define VP_REG_MTR_PROF_PAGE(n)     (3 + n)
#define VP_REG_MTR_PROF_OFFSET(n)   0x72
#define VP_REG_MTR_PROF_LENGTH      11
#define VP_REG_MTR_PROF_COUNT       2

/* Ring profiles */
#define VP_REG_RING_PROF_PAGE(n)    (6 + n)
#define VP_REG_RING_PROF_OFFSET(n)  0x72
#define VP_REG_RING_PROF_LENGTH     11
#define VP_REG_RING_PROF_COUNT      2

/* Tone profiles */
#define VP_REG_TONE_PROF_PAGE(n)    5
#define VP_REG_TONE_PROF_OFFSET(n)  (12 * n)
#define VP_REG_TONE_PROF_LENGTH     12
#define VP_REG_TONE_PROF_COUNT      10

/* Length (in words) of the longest VP profile type (currently CID profile) */
#define VP_REG_MAX_PROF_SIZE        VP_REG_CID_PROF_LENGTH

/* Command mailbox */
#define VP_REG_MB_DOWN_BUF_PAGE(first)      3
#define VP_REG_MB_DOWN_BUF_OFFSET(first)    first
#define VP_REG_MB_DOWN_BUF_SIZE             100

/* Response mailbox */
#define VP_REG_MB_UP_BUF_PAGE(first)        4
#define VP_REG_MB_UP_BUF_OFFSET(first)      first
#define VP_REG_MB_UP_BUF_SIZE               100

/* HBI Direct Page
 *
 * page offset linear  contents                         (len * cnt = tot = hex)
 * =-=-=-=-=-=-=-=-=-=-=-=-=  PAGE BOUNDARY  =-=-=-=-=-=-=-=-=-=-=-=-=-=
 *  DP    00-0A Hardware registers defined in "hbi_common.h"
 * ---------------------------------------------------------------------
 *  DP    0B
 *  DP    ::   NOT USED
 *  DP    0F
 * ----------------------------------------------------------------------
 *  DP    10   0
 *  DP    ::   :HS_STAT[0..1]                    (  2 *   1 =   2 =   2)
 *  DP    11   0
 * ----------------------------------------------------------------------
 *  DP    12   0
 *  DP    ::   :RAW_HS_STAT[0..1]                (  2 *   1 =   2 =   2)
 *  DP    13   0
 * ----------------------------------------------------------------------
 *  DP    14   0
 *  DP    ::   :GK_STAT[0..1]                    (  2 *   1 =   2 =   2)
 *  DP    15   0
 * ----------------------------------------------------------------------
 *  DP    16   0
 *  DP    ::   :THERM_FLT[0..1]                  (  2 *   1 =   2 =   2)
 *  DP    17   0
 * ----------------------------------------------------------------------
 *  DP    18   0
 *  DP    ::   :CLK_FLT[0..1]                    (  2 *   1 =   2 =   2)
 *  DP    19   0
 * ----------------------------------------------------------------------
 *  DP    1A   0
 *  DP    ::   :AC_FLT[0..1]                     (  2 *   1 =   2 =   2)
 *  DP    1B   0
 * ----------------------------------------------------------------------
 *  DP    1C   0
 *  DP    ::   :DC_FLT[0..1]                     (  2 *   1 =   2 =   2)
 *  DP    1D   0
 * ----------------------------------------------------------------------
 *  DP    1E   0
 *  DP    ::   :PBAT_FLT[0..1]                   (  2 *   1 =   2 =   2)
 *  DP    1F   0
 * ---------------------------------------------------------------------
 *  DP    20
 *  DP    ::   HBAT_FLT[0..1]                   (  2 *   1 =   2 =   2)
 *  DP    21
 * ---------------------------------------------------------------------
 *  DP    22
 *  DP    ::   LBAT_FLT[0..1]                   (  2 *   1 =   2 =   2)
 *  DP    23
 * ---------------------------------------------------------------------
 *  DP    28   REV_CODE[0]                      (  1 *   1 =   1 =   1)
 * ---------------------------------------------------------------------
 *  DP    29   TESTBUF_CUR[0..1]                (  1 *   2 =   2 =   2)
 * ---------------------------------------------------------------------
 *  DP    2B   TESTBUF_LINE[0..1]               (  1 *   2 =   2 =   2)
 * ---------------------------------------------------------------------
 *  DP    2D
 *  DP    2E   NOT USED
 *  DP    2F
 * ---------------------------------------------------------------------
 *  DP    30
 *  DP    ::   RESERVED
 *  DP    4F
 * ---------------------------------------------------------------------
 *  DP    50
 *  DP    ::   NOT USED
 *  DP    77
 * ---------------------------------------------------------------------
 *  DP    78
 *  DP    ::   RESERVED
 *  DP    7F
 * ---------------------------------------------------------------------
 *  DP    80
 *  DP    ::   0x8000 - 0x807F ALIASED THROUGHOUT THIS REGION
 *  DP    7F
 * =-=-=-=-=-=-=-=-=-=-=-=-=  PAGE BOUNDARY  =-=-=-=-=-=-=-=-=-=-=-=-=-=
 */

#define VP_REG_STAT_INPUTS_PAGE(n)   VP_DP
#define VP_REG_STAT_INPUTS_OFFSET(n) 0x10+2*n
#define VP_REG_STAT_INPUTS_LENGTH    2

/* Warning: VpGetDeviceStatus() uses VP_Reg_STAT_INPUTS(n) to map VpInputType
 * onto the following ten line status registers. */

#define VP_REG_HS_STAT_PAGE     VP_DP
#define VP_REG_HS_STAT_OFFSET   0x10
#define VP_REG_HS_STAT_LENGTH   2

#define VP_REG_RAW_HS_STAT_PAGE   VP_DP
#define VP_REG_RAW_HS_STAT_OFFSET 0x12
#define VP_REG_RAW_HS_STAT_LENGTH 2

#define VP_REG_GK_STAT_PAGE     VP_DP
#define VP_REG_GK_STAT_OFFSET   0x14
#define VP_REG_GK_STAT_LENGTH   2

#define VP_REG_THERM_FLT_PAGE   VP_DP
#define VP_REG_THERM_FLT_OFFSET 0x16
#define VP_REG_THERM_FLT_LENGTH 2

#define VP_REG_CLK_FLT_PAGE     VP_DP
#define VP_REG_CLK_FLT_OFFSET   0x18
#define VP_REG_CLK_FLT_LENGTH   2

#define VP_REG_AC_FLT_PAGE      VP_DP
#define VP_REG_AC_FLT_OFFSET    0x1A
#define VP_REG_AC_FLT_LENGTH    2

#define VP_REG_DC_FLT_PAGE      VP_DP
#define VP_REG_DC_FLT_OFFSET    0x1C
#define VP_REG_DC_FLT_LENGTH    2

#define VP_REG_PBAT_FLT_PAGE    VP_DP
#define VP_REG_PBAT_FLT_OFFSET  0x1E
#define VP_REG_PBAT_FLT_LENGTH  2

#define VP_REG_HBAT_FLT_PAGE    VP_DP
#define VP_REG_HBAT_FLT_OFFSET  0x20
#define VP_REG_HBAT_FLT_LENGTH  2

#define VP_REG_LBAT_FLT_PAGE    VP_DP
#define VP_REG_LBAT_FLT_OFFSET  0x22
#define VP_REG_LBAT_FLT_LENGTH  2

/* End of line status registers. */

#define VP_REG_REV_CODE_PAGE    VP_DP
#define VP_REG_REV_CODE_OFFSET  0x28
#define VP_REG_REV_CODE_LENGTH  1

#define VP_REG_TESTBUF_CUR_PAGE(n)   VP_DP
#define VP_REG_TESTBUF_CUR_OFFSET(n) 0x29+n
#define VP_REG_TESTBUF_CUR_LENGTH    1

#define VP_REG_TESTBUF_LINE_PAGE(n)   VP_DP
#define VP_REG_TESTBUF_LINE_OFFSET(n) 0x2B+n
#define VP_REG_TESTBUF_LINE_LENGTH    1

#define VP_REG_LINE_TERM_TYPE_PAGE(n)   VP_DP
#define VP_REG_LINE_TERM_TYPE_OFFSET(n) 0x30+n
#define VP_REG_LINE_TERM_TYPE_LENGTH    1

/* 2 words; word at the lowest address specifies the status for lines 31 through 16 */
#define VP_REG_DTMF_ENABLE_STATUS_PAGE      VP_DP
#define VP_REG_DTMF_ENABLE_STATUS_OFFSET    0x50
#define VP_REG_DTMF_ENABLE_STATUS_LENGTH    2

/* Number of remaining DTMF detection resources. */
#define VP_REG_DTMF_REMAINING_RSRC_PAGE      VP_DP
#define VP_REG_DTMF_REMAINING_RSRC_OFFSET    0x52
#define VP_REG_DTMF_REMAINING_RSRC_LENGTH    1

/* One-byte line state indicator for each line; high byte = even line number */
#define VP_REG_LINE_STATE_PAGE(n)   VP_DP
#define VP_REG_LINE_STATE_OFFSET(n) 0x53+(n>>1)
#define VP_REG_LINE_STATE_LENGTH    1
/* End definition of VCP register space */

/* Shortcut macros for arguments to VpHbiWr() and VpHbiRd() */
#define VP_REG_AC_PROF(n)       VP_REG_AC_PROF_PAGE(n),\
    VP_REG_AC_PROF_OFFSET(n),VP_REG_AC_PROF_LENGTH - 1
#define VP_REG_CAD_PROF(n)      VP_REG_CAD_PROF_PAGE(n),\
    VP_REG_CAD_PROF_OFFSET(n),VP_REG_CAD_PROF_LENGTH - 1
#define VP_REG_CID_PROF(n)      VP_REG_CID_PROF_PAGE(n),\
    VP_REG_CID_OFFSET(n),VP_REG_CID_LENGTH - 1
#define VP_REG_DC_PROF(n)       VP_REG_DC_PROF_PAGE(n),\
    VP_REG_DC_OFFSET(n),VP_REG_DC_LENGTH - 1
#define VP_REG_DEV_PROF(n)      VP_REG_DEV_PROF_PAGE(n),\
    VP_REG_DEV_PROF_OFFSET(n),VP_REG_DEV_PROF_LENGTH - 1
#define VP_REG_MTR_PROF(n)      VP_REG_MTR_PROF_PAGE(n),\
    VP_REG_MTR_PROF_OFFSET(n),VP_REG_MTR_PROF_LENGTH - 1
#define VP_REG_RING_PROF(n)     VP_REG_RING_PROF_PAGE(n),\
    VP_REG_RING_PROF_OFFSET(n),VP_REG_RING_PROF_LENGTH - 1
#define VP_REG_TONE_PROF(n)     VP_REG_TONE_PROF_PAGE(n),\
    VP_REG_TONE_PROF_OFFSET(n),VP_REG_TONE_PROF_LENGTH - 1
#define VP_REG_STAT_INPUTS(n)   VP_REG_STAT_INPUTS_PAGE(n),\
    VP_REG_STAT_INPUTS_OFFSET(n),VP_REG_STAT_INPUTS_LENGTH - 1
#define VP_REG_HS_STAT          VP_REG_HS_STAT_PAGE,\
    VP_REG_HS_STAT_OFFSET,VP_REG_HS_STAT_LENGTH - 1
#define VP_REG_RAW_HS_STAT      VP_REG_RAW_HS_STAT_PAGE,\
    VP_REG_RAW_HS_STAT_OFFSET,VP_REG_RAW_HS_STAT_LENGTH - 1
#define VP_REG_GK_STAT          VP_REG_GK_STAT_PAGE,\
    VP_REG_GK_STAT_OFFSET,VP_REG_GK_STAT_LENGTH - 1
#define VP_REG_THERM_FLT        VP_REG_THERM_FLT_PAGE,\
    VP_REG_THERM_FLT_OFFSET,VP_REG_THERM_FLT_LENGTH - 1
#define VP_REG_CLK_FLT          VP_REG_CLK_FLT_PAGE,\
    VP_REG_CLK_FLT_OFFSET,VP_REG_CLK_FLT_LENGTH - 1
#define VP_REG_AC_FLT           VP_REG_AC_FLT_PAGE,\
    VP_REG_AC_FLT_OFFSET,VP_REG_AC_FLT_LENGTH - 1
#define VP_REG_DC_FLT           VP_REG_DC_FLT_PAGE,\
    VP_REG_DC_FLT_OFFSET,VP_REG_DC_FLT_LENGTH - 1
#define VP_REG_PBAT_FLT         VP_REG_PBAT_FLT_PAGE,\
    VP_REG_PBAT_FLT_OFFSET,VP_REG_PBAT_FLT_LENGTH - 1
#define VP_REG_HBAT_FLT         VP_REG_HBAT_FLT_PAGE,\
    VP_REG_HBAT_FLT_OFFSET,VP_REG_HBAT_FLT_LENGTH - 1
#define VP_REG_LBAT_FLT         VP_REG_LBAT_FLT_PAGE,\
    VP_REG_LBAT_FLT_OFFSET,VP_REG_LBAT_FLT_LENGTH - 1
#define VP_REG_REV_CODE         VP_REG_REV_CODE_PAGE,VP_REG_REV_CODE_OFFSET,VP_REG_REV_CODE_LENGTH - 1
#define VP_REG_TESTBUF_CUR(n)   VP_REG_TESTBUF_CUR_PAGE(n),\
    VP_REG_TESTBUF_CUR_OFFSET(n),VP_REG_TESTBUF_CUR_LENGTH - 1
#define VP_REG_TESTBUF_LINE(n)  VP_REG_TESTBUF_LINE_PAGE(n),\
    VP_REG_TESTBUF_LINE_OFFSET(n),VP_REG_TESTBUF_LINE_LENGTH - 1
#define VP_REG_LINE_STATE(n)    VP_REG_LINE_STATE_PAGE(n),\
    VP_REG_LINE_STATE_OFFSET(n),VP_REG_LINE_STATE_LENGTH - 1
#define VP_REG_LINE_TERM_TYPE(n) VP_REG_LINE_TERM_TYPE_PAGE(n),\
    VP_REG_LINE_TERM_TYPE_OFFSET(n),VP_REG_LINE_TERM_TYPE_LENGTH - 1
#define VP_REG_DTMF_ENABLE_STATUS VP_REG_DTMF_ENABLE_STATUS_PAGE,\
    VP_REG_DTMF_ENABLE_STATUS_OFFSET,VP_REG_DTMF_ENABLE_STATUS_LENGTH - 1
#define VP_REG_DTMF_REMAINING_RSRC VP_REG_DTMF_REMAINING_RSRC_PAGE,\
    VP_REG_DTMF_REMAINING_RSRC_OFFSET,VP_REG_DTMF_REMAINING_RSRC_LENGTH - 1
#define VP_REG_MB_UP_BUF(first,length) VP_REG_MB_UP_BUF_PAGE(first),\
    VP_REG_MB_UP_BUF_OFFSET(first),length - 1
#define VP_REG_MB_DOWN_BUF(first,length) VP_REG_MB_DOWN_BUF_PAGE(first),\
    VP_REG_MB_DOWN_BUF_OFFSET(first),length - 1

/* Bit field definitions for select registers */
#define VP_BIT_MBOXFLAG_CMD     0x0001  /* Command mailbox flag */
#define VP_BIT_MBOXFLAG_RSP     0x0002  /* Response mailbox flag */


/*
 * Algorithm Test Buffers AMBA addresses
 * This constant defines the base addresses of the test vector buffers in VCP
 * data memory. If invoked with the "use_buffer" flag, the VCP test functions
 * extract samples from these buffers instead of getting samples from the PCM
 * highway. This mechanism allows this host to feed known data to the VCP
 * algorithms and verify the result.  The test buffer size is defined in
 * dvp_api.h (VP_TEST_BUF_MAX_SAMPLES because the user must know the maximum
 * length of test vectors allowed by the VCP.
 */
#define VP_TEST_BUF_AMBA(n) (n ? 0x2000 : 0x0000)
#define VP_NUM_TEST_BUFS    2


/* Mailbox Defines */
#define VP_MB_FUNC_INDEX            0
#define VP_MB_CONTEXT_INDEX         1
#define VP_MB_ARGLIST_INDEX         2

                            /* Mailbox Commands */
typedef enum {
    VCP_MBFUNC_SWRESET,     /* VpSoftReset() */
    VCP_MBFUNC_INITDEVICE,  /* VpInitDevice() */
    VCP_MBFUNC_INITLINE,    /* VpConfigLine() */
    VCP_MBFUNC_CALSLAC,     /* VpCalCodec() */
    VCP_MBFUNC_CALSLIC,     /* VpCalLine() */
    VCP_MBFUNC_INITRING,    /* VpInitRing() */
    VCP_MBFUNC_INITMETER,   /* VpInitMeter() */
    VCP_MBFUNC_FLUSH,       /* VpFlushEvents() */
    VCP_MBFUNC_GETLOOPCOND, /* VpGetLoopCond() */
    VCP_MBFUNC_LINESTATE,   /* VpSetLineState() */
    VCP_MBFUNC_LINETONE,    /* VpSetLineTone() */
    VCP_MBFUNC_RELAYSTATE,  /* VpSetRelayState() */
    VCP_MBFUNC_RELGAIN,     /* VpSetRelGain() */
    VCP_MBFUNC_SENDCID,     /* VpSendCid() */
    VCP_MBFUNC_METER,       /* VpStartMeter() */
    VCP_MBFUNC_SETOPTION,   /* VpSetOption() */
    VCP_MBFUNC_GETOPTION,   /* VpGetOption() */
    VCP_MBFUNC_SELFTEST,    /* VpSelfTest() */
    VCP_MBFUNC_TESTLINE,    /* VpTestLine() */
    VCP_MBFUNC_CHECKSUM,    /* VpCodeCheckSum() */
    VCP_MBFUNC_MPICMD,      /* VpMpiCmd() */
    VCP_MBFUNC_INIT_CID,    /* VpInitCid() */
    VCP_MBFUNC_CONT_CID,    /* VpContinueCid() */
    VCP_MBFUNC_RESETLINE,   /* VpInitLine() */
    VCP_MBFUNC_SEND_SIGNAL, /* VpSendSignal() */
    VCP_MBFUNC_DEVICE_IO,   /* VpDeviceIoAccess() */
    VCP_MBFUNC_ENUM_SIZE = FORCE_STANDARD_C_ENUM_SIZE /* Portability Req. */
} VcpMbFuncIdType;


#endif /* _VCP_HBI_H */




